@@ -2,7 +2,6 @@ import numpy as np
 import logging
 import itertools
 import functools
-import collections
 import time
 
 log = logging.getLogger(__name__)
@@ -14,81 +13,14 @@ from . import train
 from . import common
 from . import framing
 from . import equalizer
-
-
-class Detector(object):
-
-    COHERENCE_THRESHOLD = 0.9
-
-    CARRIER_DURATION = sum(train.prefix)
-    CARRIER_THRESHOLD = int(0.9 * CARRIER_DURATION)
-    SEARCH_WINDOW = int(0.1 * CARRIER_DURATION)
-
-    def __init__(self, config):
-        self.freq = config.Fc
-        self.omega = 2 * np.pi * self.freq / config.Fs
-        self.Nsym = config.Nsym
-        self.Tsym = config.Tsym
-        self.maxlen = config.baud  # 1 second of symbols
-
-    def run(self, samples):
-        counter = 0
-        bufs = collections.deque([], maxlen=self.maxlen)
-        for offset, buf in common.iterate(samples, self.Nsym, enumerate=True):
-            bufs.append(buf)
-
-            coeff = dsp.coherence(buf, self.omega)
-            if abs(coeff) > self.COHERENCE_THRESHOLD:
-                counter += 1
-            else:
-                counter = 0
-
-            if counter == self.CARRIER_THRESHOLD:
-                break
-        else:
-            raise ValueError('No carrier detected')
-
-        length = (self.CARRIER_THRESHOLD - 1) * self.Nsym
-        begin = offset - length
-
-        x = np.concatenate(tuple(bufs)[-self.CARRIER_THRESHOLD:-1])
-        Hc = dsp.exp_iwt(-self.omega, len(x))
-        Zc = np.dot(Hc, x) / (0.5*len(x))
-        amplitude = abs(Zc)
-        start_time = begin * self.Tsym / self.Nsym
-        log.info('Carrier detected at ~%.1f ms @ %.1f kHz:'
-                 ' coherence=%.3f%%, amplitude=%.3f',
-                 start_time * 1e3, self.freq / 1e3,
-                 np.abs(dsp.coherence(x, self.omega)) * 100, amplitude)
-
-        log.debug('Buffered %d ms of audio', len(bufs))
-
-        bufs = list(bufs)[-self.CARRIER_THRESHOLD-self.SEARCH_WINDOW:]
-        n = self.SEARCH_WINDOW + self.CARRIER_DURATION - self.CARRIER_THRESHOLD
-        trailing = list(itertools.islice(samples, n * self.Nsym))
-        bufs.append(np.array(trailing))
-
-        buf = np.concatenate(bufs)
-        offset = self.find_start(buf, self.CARRIER_DURATION*self.Nsym)
-        start_time += (offset / self.Nsym - self.SEARCH_WINDOW) * self.Tsym
-        log.debug('Carrier starts at %.3f ms', start_time * 1e3)
-
-        return itertools.chain(buf[offset:], samples), amplitude
-
-    def find_start(self, buf, length):
-        N = len(buf)
-        carrier = dsp.exp_iwt(self.omega, N)
-        z = np.cumsum(buf * carrier)
-        z = np.concatenate([[0], z])
-        correlations = np.abs(z[length:] - z[:-length])
-        return np.argmax(correlations)
+from . import detect
 
 
 class Receiver(object):
 
-    def __init__(self, config, plt=None):
+    def __init__(self, config, pylab=None):
         self.stats = {}
-        self.plt = plt or common.Dummy()
+        self.plt = pylab or common.Dummy()
         self.modem = dsp.MODEM(config.symbols)
         self.frequencies = np.array(config.frequencies)
         self.omegas = 2 * np.pi * self.frequencies / config.Fs
@@ -130,7 +62,7 @@ class Receiver(object):
         log.debug('Current phase on carrier: %.3f', last_phase)
 
         log.debug('Frequency error: %.2f ppm', freq_err * 1e6)
-        self.plt.title('Frequency drift: {:.3f} ppm'.format(freq_err * 1e6))
+        self.plt.title('Frequency drift: {0:.3f} ppm'.format(freq_err * 1e6))
         return freq_err
 
     def _train(self, sampler, order, lookahead):
@@ -172,7 +104,7 @@ class Receiver(object):
         for (i, freq), snr in zip(enumerate(self.frequencies), SNRs):
             log.debug('%5.1f kHz: SNR = %5.2f dB', freq / 1e3, snr)
             self._constellation(symbols[:, i], train_symbols[:, i],
-                                '$F_c = {} Hz$'.format(freq), index=i)
+                                '$F_c = {0} Hz$'.format(freq), index=i)
         assert error_rate == 0, error_rate
 
         return equalization_filter
@@ -222,7 +154,7 @@ class Receiver(object):
 
     def start(self, signal, gain=1.0):
         sampler = sampling.Sampler(signal, sampling.Interpolator())
-        symbols = dsp.Demux(sampler=sampler, omegas=self.omegas, Nsym=self.Nsym)
+        symbols = dsp.Demux(sampler, omegas=self.omegas, Nsym=self.Nsym)
         freq_err = self._prefix(symbols, gain=gain)
         sampler.freq -= freq_err
 
@@ -255,7 +187,7 @@ class Receiver(object):
             symbol_list = np.array(self.stats['symbol_list'])
             for i, freq in enumerate(self.frequencies):
                 self._constellation(symbol_list[i], self.modem.symbols,
-                                    '$F_c = {} Hz$'.format(freq), index=i)
+                                    '$F_c = {0} Hz$'.format(freq), index=i)
         self.plt.show()
 
     def _constellation(self, y, symbols, title, index=None):
@@ -277,24 +209,24 @@ class Receiver(object):
         self.plt.title(title)
 
 
-def main(args):
-    config = args.config
-    reader = stream.Reader(args.input, data_type=common.loads)
+def main(config, src, dst, pylab=None):
+    reader = stream.Reader(src, data_type=common.loads)
     signal = itertools.chain.from_iterable(reader)
 
-    skipped = common.take(signal, args.skip)
-    log.debug('Skipping %.3f seconds', len(skipped) / float(config.baud))
+    to_skip = int(config.skip_start * config.Fs)
+    log.debug('Skipping %.3f seconds', config.skip_start)
+    common.take(signal, to_skip)
 
     reader.check = common.check_saturation
 
-    detector = Detector(config=config)
-    receiver = Receiver(config=config, plt=args.plot)
+    detector = detect.Detector(config=config)
+    receiver = Receiver(config=config, pylab=pylab)
     success = False
     try:
         log.info('Waiting for carrier tone: %.1f kHz', config.Fc / 1e3)
         signal, amplitude = detector.run(signal)
         receiver.start(signal, gain=1.0/amplitude)
-        receiver.run(args.output)
+        receiver.run(dst)
         success = True
     except Exception:
         log.exception('Decoding failed')
